from __future__ import absolute_import, division, print_function

import paayes


class TestUsageRecordSummary(object):
    def test_is_listable(self, request_mock):
        resource = paayes.SubscriptionItem.retrieve("si_123")
        usage_record_summaries = resource.usage_record_summaries()
        request_mock.assert_requested(
            "get", "/api/v1/subscription_items/si_123/usage_record_summaries"
        )
        assert isinstance(usage_record_summaries.data, list)
        assert isinstance(
            usage_record_summaries.data[0], paayes.UsageRecordSummary
        )



from __future__ import absolute_import, division, print_function

# flake8: noqa

from paayes.api_resources.error_object import ErrorObject, OAuthErrorObject
from paayes.api_resources.list_object import ListObject

from paayes.api_resources import billing_portal
from paayes.api_resources import checkout
from paayes.api_resources import identity
from paayes.api_resources import issuing
from paayes.api_resources import radar
from paayes.api_resources import reporting
from paayes.api_resources import sigma
from paayes.api_resources import terminal

from paayes.api_resources.account import Account
from paayes.api_resources.account_link import AccountLink
from paayes.api_resources.alipay_account import AlipayAccount
from paayes.api_resources.apple_pay_domain import ApplePayDomain
from paayes.api_resources.application_fee import ApplicationFee
from paayes.api_resources.application_fee_refund import ApplicationFeeRefund
from paayes.api_resources.balance import Balance
from paayes.api_resources.balance_transaction import BalanceTransaction
from paayes.api_resources.bank_account import BankAccount
from paayes.api_resources.bitcoin_receiver import BitcoinReceiver
from paayes.api_resources.bitcoin_transaction import BitcoinTransaction
from paayes.api_resources.capability import Capability
from paayes.api_resources.card import Card
from paayes.api_resources.charge import Charge
from paayes.api_resources.country_spec import CountrySpec
from paayes.api_resources.coupon import Coupon
from paayes.api_resources.credit_note import CreditNote
from paayes.api_resources.credit_note_line_item import CreditNoteLineItem
from paayes.api_resources.customer import Customer
from paayes.api_resources.customer_balance_transaction import (
    CustomerBalanceTransaction,
)
from paayes.api_resources.dispute import Dispute
from paayes.api_resources.ephemeral_key import EphemeralKey
from paayes.api_resources.event import Event
from paayes.api_resources.exchange_rate import ExchangeRate
from paayes.api_resources.file import File
from paayes.api_resources.file import FileUpload
from paayes.api_resources.file_link import FileLink
from paayes.api_resources.invoice import Invoice
from paayes.api_resources.invoice_item import InvoiceItem
from paayes.api_resources.invoice_line_item import InvoiceLineItem
from paayes.api_resources.issuer_fraud_record import IssuerFraudRecord
from paayes.api_resources.line_item import LineItem
from paayes.api_resources.login_link import LoginLink
from paayes.api_resources.mandate import Mandate
from paayes.api_resources.order import Order
from paayes.api_resources.order_return import OrderReturn
from paayes.api_resources.payment_intent import PaymentIntent
from paayes.api_resources.payment_method import PaymentMethod
from paayes.api_resources.payout import Payout
from paayes.api_resources.person import Person
from paayes.api_resources.plan import Plan
from paayes.api_resources.price import Price
from paayes.api_resources.product import Product
from paayes.api_resources.promotion_code import PromotionCode
from paayes.api_resources.quote import Quote
from paayes.api_resources.recipient import Recipient
from paayes.api_resources.recipient_transfer import RecipientTransfer
from paayes.api_resources.refund import Refund
from paayes.api_resources.reversal import Reversal
from paayes.api_resources.review import Review
from paayes.api_resources.setup_attempt import SetupAttempt
from paayes.api_resources.setup_intent import SetupIntent
from paayes.api_resources.sku import SKU
from paayes.api_resources.source import Source
from paayes.api_resources.source_transaction import SourceTransaction
from paayes.api_resources.subscription import Subscription
from paayes.api_resources.subscription_item import SubscriptionItem
from paayes.api_resources.subscription_schedule import SubscriptionSchedule
from paayes.api_resources.tax_code import TaxCode
from paayes.api_resources.tax_id import TaxId
from paayes.api_resources.tax_rate import TaxRate
from paayes.api_resources.three_d_secure import ThreeDSecure
from paayes.api_resources.token import Token
from paayes.api_resources.topup import Topup
from paayes.api_resources.transfer import Transfer
from paayes.api_resources.usage_record import UsageRecord
from paayes.api_resources.usage_record_summary import UsageRecordSummary
from paayes.api_resources.webhook_endpoint import WebhookEndpoint



from __future__ import absolute_import, division, print_function

import paayes


class TestSetupAttempt(object):
    def test_is_listable(self, request_mock):
        resources = paayes.SetupAttempt.list(setup_intent="seti_123")
        request_mock.assert_requested("get", "/api/v1/setup_attempts")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], paayes.SetupAttempt)



from __future__ import absolute_import, division, print_function

from paayes import api_requestor, util
from paayes.api_resources.abstract.api_resource import APIResource


class ListableAPIResource(APIResource):
    @classmethod
    def auto_paging_iter(cls, *args, **params):
        return cls.list(*args, **params).auto_paging_iter()

    @classmethod
    def list(
        cls, api_key=None, paayes_version=None, paayes_account=None, **params
    ):
        requestor = api_requestor.APIRequestor(
            api_key,
            api_base=cls.api_base(),
            api_version=paayes_version,
            account=paayes_account,
        )
        url = cls.class_url()
        response, api_key = requestor.request("get", url, params)
        paayes_object = util.convert_to_paayes_object(
            response, api_key, paayes_version, paayes_account
        )
        paayes_object._retrieve_params = params
        return paayes_object



from __future__ import absolute_import, division, print_function

import paayes


class TestBalance(object):
    def test_is_retrievable(self, request_mock):
        resource = paayes.Balance.retrieve()
        request_mock.assert_requested("get", "/api/v1/balance")
        assert isinstance(resource, paayes.Balance)



from __future__ import absolute_import, division, print_function

import pytest

import paayes


TEST_RESOURCE_ID = "cbtxn_123"


class TestCustomerBalanceTransaction(object):
    def construct_resource(self):
        tax_id_dict = {
            "id": TEST_RESOURCE_ID,
            "object": "customer_balance_transaction",
            "customer": "cus_123",
        }
        return paayes.CustomerBalanceTransaction.construct_from(
            tax_id_dict, paayes.api_key
        )

    def test_has_instance_url(self, request_mock):
        resource = self.construct_resource()
        assert (
            resource.instance_url()
            == "/api/v1/customers/cus_123/balance_transactions/%s"
            % TEST_RESOURCE_ID
        )

    def test_is_not_retrievable(self, request_mock):
        with pytest.raises(NotImplementedError):
            paayes.CustomerBalanceTransaction.retrieve(TEST_RESOURCE_ID)



from __future__ import absolute_import, division, print_function

# flake8: noqa

from paayes.api_resources.issuing.authorization import Authorization
from paayes.api_resources.issuing.card import Card
from paayes.api_resources.issuing.card_details import CardDetails
from paayes.api_resources.issuing.cardholder import Cardholder
from paayes.api_resources.issuing.dispute import Dispute
from paayes.api_resources.issuing.transaction import Transaction



from __future__ import absolute_import, division, print_function

import os

import paayes
from flask import Flask, request, redirect


paayes.api_key = os.environ.get("PAAYES_SECRET_KEY")
paayes.client_id = os.environ.get("PAAYES_CLIENT_ID")

app = Flask(__name__)


@app.route("/")
def index():
    return '<a href="/authorize">Connect with Paayes</a>'


@app.route("/authorize")
def authorize():
    url = paayes.OAuth.authorize_url(scope="read_only")
    return redirect(url)


@app.route("/oauth/callback")
def callback():
    code = request.args.get("code")
    try:
        resp = paayes.OAuth.token(grant_type="authorization_code", code=code)
    except paayes.oauth_error.OAuthError as e:
        return "Error: " + str(e)

    return """
<p>Success! Account <code>{paayes_user_id}</code> is connected.</p>
<p>Click <a href="/deauthorize?paayes_user_id={paayes_user_id}">here</a> to
disconnect the account.</p>
""".format(
        paayes_user_id=resp["paayes_user_id"]
    )


@app.route("/deauthorize")
def deauthorize():
    paayes_user_id = request.args.get("paayes_user_id")
    try:
        paayes.OAuth.deauthorize(paayes_user_id=paayes_user_id)
    except paayes.oauth_error.OAuthError as e:
        return "Error: " + str(e)

    return """
<p>Success! Account <code>{paayes_user_id}</code> is disconnected.</p>
<p>Click <a href="/">here</a> to restart the OAuth flow.</p>
""".format(
        paayes_user_id=paayes_user_id
    )


if __name__ == "__main__":
    app.run(port=int(os.environ.get("PORT", 5000)))



from __future__ import absolute_import, division, print_function

import paayes


class TestSourceTransaction(object):
    def test_is_listable(self, request_mock):
        source = paayes.Source.construct_from(
            {"id": "src_123", "object": "source"}, paayes.api_key
        )
        source_transactions = source.source_transactions()
        request_mock.assert_requested(
            "get", "/api/v1/sources/src_123/source_transactions"
        )
        assert isinstance(source_transactions.data, list)
        assert isinstance(
            source_transactions.data[0], paayes.SourceTransaction
        )



from __future__ import absolute_import, division, print_function

import paayes


TEST_RESOURCE_ID = "ich_123"


class TestCardholder(object):
    def test_is_creatable(self, request_mock):
        resource = paayes.issuing.Cardholder.create(
            billing={
                "address": {
                    "city": "city",
                    "country": "US",
                    "line1": "line1",
                    "postal_code": "postal_code",
                }
            },
            name="Jenny Rosen",
            type="individual",
        )
        request_mock.assert_requested("post", "/api/v1/issuing/cardholders")
        assert isinstance(resource, paayes.issuing.Cardholder)

    def test_is_listable(self, request_mock):
        resources = paayes.issuing.Cardholder.list()
        request_mock.assert_requested("get", "/api/v1/issuing/cardholders")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], paayes.issuing.Cardholder)

    def test_is_modifiable(self, request_mock):
        resource = paayes.issuing.Cardholder.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        request_mock.assert_requested(
            "post", "/api/v1/issuing/cardholders/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, paayes.issuing.Cardholder)

    def test_is_retrievable(self, request_mock):
        resource = paayes.issuing.Cardholder.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/api/v1/issuing/cardholders/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, paayes.issuing.Cardholder)

    def test_is_saveable(self, request_mock):
        resource = paayes.issuing.Cardholder.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        cardholder = resource.save()
        request_mock.assert_requested(
            "post", "/api/v1/issuing/cardholders/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, paayes.issuing.Cardholder)
        assert resource is cardholder



from __future__ import absolute_import, division, print_function

# flake8: noqa

from paayes.api_resources.billing_portal.configuration import Configuration
from paayes.api_resources.billing_portal.session import Session



from __future__ import absolute_import, division, print_function

from paayes.api_resources.abstract import CreateableAPIResource
from paayes.api_resources.abstract import ListableAPIResource
from paayes.api_resources.abstract import UpdateableAPIResource


class Price(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "price"



from __future__ import absolute_import, division, print_function

import paayes


TEST_RESOURCE_ID = "activity.summary.1"


class TestReportType(object):
    def test_is_listable(self, request_mock):
        resources = paayes.reporting.ReportType.list()
        request_mock.assert_requested("get", "/api/v1/reporting/report_types")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], paayes.reporting.ReportType)

    def test_is_retrievable(self, request_mock):
        resource = paayes.reporting.ReportType.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/api/v1/reporting/report_types/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, paayes.reporting.ReportType)



from __future__ import absolute_import, division, print_function

import paayes


TEST_RESOURCE_ID = "promo_123"


class TestPromotionCode(object):
    def test_is_listable(self, request_mock):
        resources = paayes.PromotionCode.list()
        request_mock.assert_requested("get", "/api/v1/promotion_codes")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], paayes.PromotionCode)

    def test_is_retrievable(self, request_mock):
        resource = paayes.PromotionCode.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/api/v1/promotion_codes/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, paayes.PromotionCode)

    def test_is_creatable(self, request_mock):
        resource = paayes.PromotionCode.create(coupon="co_123", code="MYCODE")
        request_mock.assert_requested("post", "/api/v1/promotion_codes")
        assert isinstance(resource, paayes.PromotionCode)

    def test_is_saveable(self, request_mock):
        resource = paayes.PromotionCode.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        resource.save()
        request_mock.assert_requested(
            "post", "/api/v1/promotion_codes/%s" % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self, request_mock):
        resource = paayes.PromotionCode.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        request_mock.assert_requested(
            "post", "/api/v1/promotion_codes/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, paayes.PromotionCode)



from __future__ import absolute_import, division, print_function

from paayes.paayes_object import PaayesObject


# This resource can only be instantiated when expanded on a BalanceTransaction
class RecipientTransfer(PaayesObject):
    OBJECT_NAME = "recipient_transfer"
